@@ -20,16 +20,30 @@ $me $Ganglia::Gmetric::PP::VERSION
 
 Usage: $me [OPTIONS]...
 
+  --[no-]pp-client           Use Ganglia::Gmetric::PP for talking to gmond.
+                             Default off, which requires a working gmetric
+                             (see --gmetric below)
+
   -h, --remote-host=STRING   Remote host where gmond is running. Default localhost
   -p, --remote-port=INT      Remote port where gmond is running. Default 8649
+
+      --gmetric=PATH         Path to gmetric(1) binary. -h and -p options are
+                             ignored; gmetric usse its own gmond.conf for host
+                             settings.
+
   -H, --listen-host=STRING   Local interface to listen on. Default 0.0.0.0
   -P, --listen-port=INT      Local UDP port to listen on. Default 18649
   -u, --unit-suffix=STRING   Suffix to append to gmetric units. Default "/s"
   -m, --metric-suffix=STRING Suffix to append to metric names. Default "_rate"
   -n, --period=INT           Time period in seconds between aggregations. Default 60
+
   -N, --multiplier=FLOAT     Amount to multiply output values by. Default 1
-                              (e.g., use "-N 3600 -u /hour" to emit per-hour metrics).
-  -f, --[no]-floating        Always use "double" type instead of original metrics' types. Default on
+                             (e.g., use "-N 3600 -u /hour" to emit per-hour
+                             metrics).
+
+  -f, --[no]-floating        Always use "double" type instead of original metrics'
+                             types. Default on
+
   -d, --daemon               Run in daemon mode.
   -U, --user                 User to drop to (only if started as root)
   -F, --pidfile=FILE         File to write PID to in daemon mode.
@@ -41,8 +55,10 @@ EOU
 
 Getopt::Long::Configure('no_ignore_case');
 GetOptions(
+    'pp-client!'        => \(my $use_pp_client  = 0),
     'h|remote-host=s'   => \(my $remote_host    = '127.0.0.1'),
     'p|remote-port=i'   => \(my $remote_port    = 8649),
+    'gmetric=s'         => \(my $gmetric),
     'H|listen-host=s'   => \(my $listen_host    = '0.0.0.0'),
     'P|listen-port=i'   => \(my $listen_port    = 18649),
     'u|unit-suffix=s'   => \(my $units_suffix   = '/s'),
@@ -57,8 +73,23 @@ GetOptions(
     'help!'             => \(my $help),
 ) || usage;
 
+my $emitter;
+if ($use_pp_client) {
+    $emitter = Ganglia::Gmetric::PP->new(
+        host => $remote_host,
+        port => $remote_port,
+    );
+}
+else {
+    unless ($gmetric) {
+        $gmetric = qx(which gmetric);
+        die "can't find gmetric\n" unless $gmetric;
+        chomp $gmetric;
+    }
+    die "can't run gmetric\n" unless -x $gmetric;
+}
+
 usage if $help;
-usage('--remote-host needed') unless defined $remote_host;
 
 my $use_anyevent;
 if (eval "use AnyEvent; 1") {
@@ -84,11 +115,6 @@ if ($pidfile && open my $pid_fh, '>', $pidfile) {
 }
 END { unlink $pidfile if $pidfile }
 
-my $emitter = Ganglia::Gmetric::PP->new(
-    host => $remote_host,
-    port => $remote_port,
-);
-
 # udp server socket
 my $gmond = Ganglia::Gmetric::PP->new(listen_host => $listen_host, listen_port => $listen_port);
 
@@ -135,6 +161,8 @@ else {
     Danga::Socket->AddOtherFds(fileno($gmond), \&handle);
 }
 
+$SIG{CHLD} = 'IGNORE';
+
 # periodically aggregate collected samples and re-emit to target gmond
 my $timer;
 sub aggregator {
@@ -162,7 +190,23 @@ sub aggregator {
             $aggregate[METRIC_INDEX_VALUE] *= $multiplier;
             $aggregate[METRIC_INDEX_TMAX] = $period;
 
-            $emitter->send(@aggregate);
+            if ($use_pp_client) {
+                $emitter->send(@aggregate);
+            }
+            else {
+                my $pid = fork;
+                if (!defined $pid) {
+                    warn "fork failed: $!";
+                    return;
+                }
+                elsif (!$pid) {
+                    my %gmetric_args;
+                    @gmetric_args{qw/ --type --name --value --units --slope --tmax --dmax /} = @aggregate;
+                    $debug && warn "Running $gmetric @{[%gmetric_args]}\n";
+                    exec $gmetric, %gmetric_args;
+                    die "exec failed: $!\n";
+                }
+            }
 
             $debug && warn Data::Dumper->Dump([\@aggregate], [$metric]);
         }
